# md_notes
#
# This file was automatically generated by APIMATIC v2.0
# ( https://apimatic.io ).

require 'base64'

module MdNotes
  # Utility class for OAuth 2 authorization and token management.
  class OAuth2
    def self.auth_controller(config)
      OAuthAuthorizationController.new config
    end

    # Add OAuth2 authentication to the http request.
    # @param [HttpRequest] The HttpRequest object to which authentication will
    # be added.
    def self.apply(config, http_request)
      check_auth config.o_auth_token
      token = config.o_auth_token.access_token
      http_request.headers['Authorization'] = "Bearer #{token}"
    end

    # Get an OAuth token that you must then set in your Configuration object
    # to authorize subsequent calls.
    # @param [String | Array of String] Any scopes for the authentication token.
    # @param [Hash] Any additional form parameters.
    def self.authorize(config, scope: nil, additional_params: nil)
      token = auth_controller(config).request_token(
        build_basic_auth_header(config),
        config.o_auth_username,
        config.o_auth_password,
        scope: scope ? Array(scope).compact.join(' ') : nil,
        _field_parameters: additional_params
      )
      token.expiry = (Time.now.to_i + token.expires_in.to_i) if token.expires_in
      token
    end

    # Builds the basic auth header for endpoints in the
    # OAuth Authorization Controller.
    def self.build_basic_auth_header(config)
      value = "#{config.o_auth_client_id}:" \
              "#{config.o_auth_client_secret}"
      encoded = Base64.strict_encode64(value)
      "Basic #{encoded}"
    end

    # Checks if OAuth token is valid.
    def self.check_auth(token)
      # Check if OAuth token exists.
      if token.nil?
        raise 'Client is not authorized. An OAuth token is needed to ' \
              'make API calls.'
      # Check if OAuth token has expired.
      elsif token.expiry && token.expiry < Time.now.to_i
        raise 'The OAuth token has expired. Please refresh the existing token' \
              ' or generate a new one.'
      end
    end

    # Refreshes OAuth token and returns it. You must then set it in your
    # Configuration object.
    # @param [String | Array of String] Any scopes for the new authentication
    # token.
    # @param [Hash] Any additional form parameters.
    def self.refresh_token(config, scope: nil, additional_params: nil)
      token = auth_controller(config).refresh_token(
        build_basic_auth_header(config),
        config.o_auth_token.refresh_token,
        scope: scope ? Array(scope).compact.join(' ') : nil,
        _field_parameters: additional_params
      )
      token.expiry = (Time.now.to_i + token.expires_in.to_i) if token.expires_in
      token
    end
  end
end
